from dataclasses import dataclass

import boto3


@dataclass
class FailedSecurityChecks:
    error_checks: list
    warning_checks: list


class TrustedAdvisor:
    def __init__(self):
        self._support_client = boto3.client("support", region_name="us-east-1")

    def describe_checks(self):
        """
        Prints information about Trusted Advisor Security Checks

        :return: None
        """
        try:
            response = self._support_client.describe_trusted_advisor_checks(language="en")
            for check in response.get("checks"):
                if check.get("category") == "security":
                    print(f"{check.get('id')} - {check.get('name')} - {check.get('metadata')}")
        except Exception as e:
            print(e)

    def obtain_security_checks(self) -> list:
        """
        Returns the entire list of Trusted Advisor Security checks

        :return: list
        """
        security_checks = []
        try:
            response = self._support_client.describe_trusted_advisor_checks(language="en")
            for check in response.get("checks"):
                if check.get("category") == "security":
                    security_checks.append(check)
        except Exception as e:
            print(e)
        return security_checks

    def describe_check_by_check_id(self, check_id: str):
        """
        Prints the response of describing a given check by its id

        :param check_id: Id of the check
        :return: None
        """
        try:
            response = self._support_client.describe_trusted_advisor_check_result(checkId=check_id, language="en")
            print(response)
        except Exception as e:
            print(e)

    def obtain_failed_security_checks(self) -> FailedSecurityChecks:
        """
        Returns a list of all failed with error Trusted Advisor Security Checks

        :return: None
        """
        error_security_checks = []
        warning_security_checks = []
        for sec_check in self.obtain_security_checks():
            try:
                response = self._support_client.describe_trusted_advisor_check_result(
                    checkId=sec_check.get("id"), language="en"
                )
                check_status = response.get("result").get("status")
                if check_status in ["error"]:
                    error_security_checks.append(sec_check)
                if check_status in ["warning"]:
                    warning_security_checks.append(sec_check)
            except Exception as e:
                print(e)

        return FailedSecurityChecks(error_checks=error_security_checks, warning_checks=warning_security_checks)

    def show_failed_security_checks(self):
        """
        Prints failed Trusted Advisor Security Checks

        :return: None
        """
        failed_checks = self.obtain_failed_security_checks()
        print("ERROR")
        for sec_check in failed_checks.error_checks:
            print(sec_check.get("name"))
        print("WARNING")
        for sec_check in failed_checks.warning_checks:
            print(sec_check.get("name"))


def describe_severity_levels():
    """
    Prints the response of describing the security levels for the account

    :return: None
    """
    client = boto3.client("support")
    response = client.describe_severity_levels(language="en")
    print(response)



import boto3


def get_current_account_id():
    account = boto3.client("sts").get_caller_identity().get("Account")
    return account


def get_current_user_id():
    user_id = boto3.client("sts").get_caller_identity().get("UserId")
    return user_id


def get_current_arn():
    arn = boto3.client("sts").get_caller_identity().get("Arn")
    return arn


def get_current_assumed_role():
    arn = get_current_arn()
    assumed_role = arn.split("/")[1]
    return assumed_role


def get_current_caller_identity():
    caller_identity = boto3.client("sts").get_caller_identity()
    return caller_identity



import boto3


def show_all_available_regions():
    """
    Prints current available regions

    :return: None
    """
    client = boto3.client("ec2")
    for region in client.describe_regions()["Regions"]:
        print(region["RegionName"])


def show_all_regions():
    """
    Prints current available regions

    :return: None
    """
    client = boto3.client("ec2")
    for region in client.describe_regions(AllRegions=True)["Regions"]:
        print(region["RegionName"])



import inquirer

from AWholeS.ec2 import show_all_regions, show_all_available_regions
from AWholeS.sts_calls import (
    get_current_arn,
    get_current_assumed_role,
    get_current_account_id,
)
from AWholeS.trusted_advisor_security import TrustedAdvisor


def menu() -> None:
    """
    Shows the menu with current options of the app.
    Each option should call a function besides exit.
    :return: None
    """
    last_choice = None
    while True:
        choice = inquirer.list_input(
            "What do you want to run?",
            choices=["EC2", "STS", "Trusted Advisor", "exit"],
            default=last_choice,
        )
        if choice == "EC2":
            print("Showing all regions:")
            show_all_regions()
            print("\n")
            print("Showing current available regions:")
            show_all_available_regions()
            print("\n")
        elif choice == "STS":
            print("Your current ARN is:")
            print(get_current_arn())
            print("\n")
            print("Your current AssumedRole is:")
            print(get_current_assumed_role())
            print("\n")
        elif choice == "Trusted Advisor":
            ta = TrustedAdvisor()
            print(f"Current Security failures for your account {get_current_account_id()}:")
            ta.show_failed_security_checks()
            print("\n")
        elif choice == "exit":
            return
        last_choice = choice


# If this doesn't find modules, use:
# export PYTHONPATH=$PYTHONPATH:.
# Ref: https://stackoverflow.com/questions/37233140/python-module-not-found
if __name__ == "__main__":
    print("Running AWholeS...")
    menu()
